<?php
if( ! class_exists( 'cjaddons_supportezzy_gumroad' ) ) {
	class cjaddons_supportezzy_gumroad {
		private static $instance;
		public $helpers;

		public static function getInstance() {
			if( ! isset( self::$instance ) ) {
				self::$instance = new self();
			}

			return self::$instance;
		}

		public function __construct() {
			$this->helpers = cjaddons_helpers::getInstance();

			add_action( 'admin_init', array($this, 'importProducts') );
		}

		public function importProducts() {
			if( isset( $_GET['gumroad-sync'] ) && $_GET['gumroad-sync'] == 'products' ) {
				$products = $this->gumroadApi( 'get', 'https://api.gumroad.com/v2/products/' );
				$products = json_decode( $products, 1 );
				$terms = array();
				if( isset( $products['products'] ) ) {
					foreach( $products['products'] as $key => $product ) {
						if( $product['published'] ) {
							$term_id = 'gumroad-' . $product['custom_permalink'];
							$term_name = $product['name'] . ' (Gumroad)';
							$terms[ $term_id ] = $term_name;
						}
					}
				}

				if( ! empty( $terms ) ) {
					foreach( $terms as $term_slug => $term_name ) {
						$check_existing = get_term_by( 'slug', $term_slug, 'cjsupport_products' );
						if( ! $check_existing ) {
							wp_insert_term( $term_name, 'cjsupport_products', array('slug' => $term_slug) );
						} else {
							wp_update_term( $check_existing->term_id, 'cjsupport_products', array('name' => $term_name) );
						}
					}
				}

				$location = admin_url( 'edit-tags.php?taxonomy=cjsupport_products&post_type=cjsupport' );
				wp_redirect( $location );
				die();
			}
		}

		public function gumroadApi( $method = 'get', $endpoint, $post_data = array() ) {
			$result = array();
			$headers = array();
			$gumroad_token = $this->helpers->savedOption( 'cjsupport_gumroad_token' );
			$data = array(
				'access_token' => $gumroad_token
			);
			$data = array_merge( $data, $post_data );
			if( $method == 'get' ) {
				$result = $this->helpers->wpRemoteGet( $endpoint, $data, $headers );
			}
			if( $method == 'post' ) {
				$result = $this->helpers->wpRemotePost( $endpoint, $data, $headers );
			}

			return $result;
		}

		public function verifyLicense( $product_permalink, $license_key ) {
			$data = array();
			$result = $this->gumroadApi( 'post', 'https://api.gumroad.com/v2/licenses/verify', array(
				'product_permalink' => $product_permalink,
				'license_key' => $license_key,
			) );
			$result = json_decode( $result, 1 );

			if( ! $result['success'] ) {
				return array('error' => $result['message']);
			}

			if(isset($result['purchase']['subscription_cancelled_at']) && ! is_null( $result['purchase']['subscription_cancelled_at'] ) ) {
				return array('error' => sprintf( __( 'Subscription canceled at<br> %s.', 'addon-supportezzy' ), date( 'Y-m-d H:i A', strtotime( $result['purchase']['subscription_cancelled_at'] ) ) ));
			}
			if(isset($result['purchase']['subscription_failed_at']) && ! is_null( $result['purchase']['subscription_failed_at'] ) ) {
				return array('error' => sprintf( __( 'Subscription failed at<br> %s.', 'addon-supportezzy' ), date( 'Y-m-d H:i A', strtotime( $result['purchase']['subscription_failed_at'] ) ) ));
			}

			if( isset($result['purchase']['chargebacked']) && $result['purchase']['chargebacked'] && isset($result['purchase']['refunded']) && $result['purchase']['refunded'] ) {
				return array('error' => __( 'Refund for this product has been processed.', 'addon-supportezzy' ));
			}

			return array('success' => $result['purchase']);
		}

	}

	cjaddons_supportezzy_gumroad::getInstance();
}